@@ -16,10 +16,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: mimedefang.c,v 1.226 2005/06/13 15:22:41 dfs Exp $";
-
-/* Define this to work around an M$ Outlook bug! */
-/* #define CONVERT_EMBEDDED_CRS_IN_HEADERS 1 */
+"$Id: mimedefang.c,v 1.242 2006/01/28 01:59:43 dfs Exp $";
 
 #define _BSD_SOURCE 1
 
@@ -110,6 +107,9 @@ int LogTimes = 0;
 /* Default Backlog for "listen" */
 static int Backlog = -1;
 
+/* Strip out bare CR characters? */
+static int StripBareCR = 0;
+
 /* Allow new connections to queue? */
 static int AllowNewConnectionsToQueue = 0;
 
@@ -162,6 +162,8 @@ static sfsistat mfclose(SMFICTX *ctx);
 static int do_sm_quarantine(SMFICTX *ctx, char const *reason);
 static void remove_working_directory(struct privdata *data);
 
+static char const *SpoolDir = NULL;
+static char const *NoDeleteDir = NULL;
 static char const *MultiplexorSocketName = NULL;
 
 static int set_reply(SMFICTX *ctx, char const *first, char const *code, char const *dsn, char const *reply);
@@ -183,6 +185,9 @@ static pthread_mutex_t MkdirMutex = PTHREAD_MUTEX_INITIALIZER;
 /* Do relay check? */
 static int doRelayCheck = 0;
 
+/* Do HELO check? */
+static int doHeloCheck = 0;
+
 /* Do sender check? */
 static int doSenderCheck = 0;
 
@@ -195,9 +200,12 @@ static int keepFailedDirectories = 0;
 /* Protect mkdir() with mutex? */
 static int protectMkdirWithMutex = 0;
 
+/* Number of scanning slaves reserved for connection from loopback */
+static int slavesReservedForLoopback = -1;
+
 static void set_dsn(SMFICTX *ctx, char *buf2, int code);
 
-#define NO_DELETE_DIR SPOOLDIR "/DO-NOT-DELETE-WORK-DIRS"
+#define NO_DELETE_NAME "/DO-NOT-DELETE-WORK-DIRS"
 
 #ifdef ENABLE_DEBUGGING
 /**********************************************************************
@@ -381,19 +389,39 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     /* Delete any existing context data */
     mfclose(ctx);
 
-    /* If too many running filters, reject connection at this phase */
+    /* If too many running filters, reject connection at this phase.
+       However, if slavesReservedForLoopback >= 0, accept or queue
+       connections from localhost
+     */
     if (!AllowNewConnectionsToQueue) {
+	int is_local;
+	int required_slaves;
 	int n = MXCheckFreeSlaves(MultiplexorSocketName);
-	if (n == 0) {
-	    syslog(LOG_WARNING, "mfconnect: No free slaves");
-	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
-	    return SMFIS_TEMPFAIL;
-	}
 	if (n < 0) {
 	    syslog(LOG_WARNING, "mfconnect: Error communicating with multiplexor");
 	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
 	    return SMFIS_TEMPFAIL;
 	}
+
+	is_local = is_localhost(sa);
+
+	if (slavesReservedForLoopback >= 0) {
+	    if (is_local) {
+		required_slaves = 0;
+	    } else {
+		required_slaves = slavesReservedForLoopback;
+	    }
+	} else {
+	    required_slaves = 0;
+	}
+	if (n <= required_slaves) {
+	    if (slavesReservedForLoopback < 0 ||
+		! is_local) {
+		syslog(LOG_WARNING, "mfconnect: No free slaves: Need %d, found %d", required_slaves+1, n);
+		DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
+		return SMFIS_TEMPFAIL;
+	    }
+	}
     }
 
     data = malloc_with_log(sizeof *data);
@@ -498,7 +526,7 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 	char buf2[SMALLBUF];
 	int n = MXRelayOK(MultiplexorSocketName, buf2, data->hostip,
 			  data->hostname);
-	if (n == 0) {
+	if (n == MD_REJECT) {
 	    /* Can't call smfi_setreply from connect callback */
 	    /* set_dsn(ctx, buf2, 5); */
 	    /* We reject connections from this relay */
@@ -506,20 +534,20 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_REJECT");
 	    return SMFIS_REJECT;
 	}
-	if (n < 0) {
+	if (n <= MD_TEMPFAIL) {
 	    /* Can't call smfi_setreply from connect callback */
 	    /* set_dsn(ctx, buf2, 4); */
 	    cleanup(ctx);
 	    DEBUG_EXIT("mfconnect", __LINE__, "SMFIS_TEMPFAIL");
 	    return SMFIS_TEMPFAIL;
 	}
-	if (n == 2) {
+	if (n == MD_ACCEPT_AND_NO_MORE_FILTERING) {
 	    /* Can't call smfi_setreply from connect callback */
 	    /* set_dsn(ctx, buf2, 2); */
 	    cleanup(ctx);
 	    return SMFIS_ACCEPT;
 	}
-	if (n == 3) {
+	if (n == MD_DISCARD) {
 	    /* Can't call smfi_setreply from connect callback */
 	    /* set_dsn(ctx, buf2, 2); */
 	    cleanup(ctx);
@@ -554,6 +582,40 @@ helo(SMFICTX *ctx, char *helohost)
 	data->heloArg = NULL;
     }
     data->heloArg = strdup_with_log(helohost);
+    if (doHeloCheck) {
+	char buf2[SMALLBUF];
+	int n = MXHeloOK(MultiplexorSocketName, buf2, data->hostip,
+			 data->hostname, data->heloArg);
+	if (n == MD_REJECT) {
+	    set_dsn(ctx, buf2, 5);
+	    cleanup(ctx);
+	    DEBUG_EXIT("helo", __LINE__, "SMFIS_REJECT");
+	    return SMFIS_REJECT;
+	}
+	if (n <= MD_TEMPFAIL) {
+	    set_dsn(ctx, buf2, 4);
+	    cleanup(ctx);
+	    DEBUG_EXIT("helo", __LINE__, "SMFIS_TEMPFAIL");
+	    return SMFIS_TEMPFAIL;
+	}
+	if (n == MD_ACCEPT_AND_NO_MORE_FILTERING) {
+	    set_dsn(ctx, buf2, 2);
+	    cleanup(ctx);
+	    DEBUG_EXIT("helo", __LINE__, "SMFIS_ACCEPT");
+	    return SMFIS_ACCEPT;
+	}
+	if (n == MD_DISCARD) {
+	    set_dsn(ctx, buf2, 2);
+	    cleanup(ctx);
+	    DEBUG_EXIT("helo", __LINE__, "SMFIS_DISCARD");
+	    return SMFIS_DISCARD;
+	}
+	if (n == MD_CONTINUE) {
+	    /* Called only in case we need to delay */
+	    set_dsn(ctx, buf2, 2);
+	}
+    }
+    DEBUG_EXIT("helo", __LINE__, "SMFIS_CONTINUE");
     return SMFIS_CONTINUE;
 }
 
@@ -634,7 +696,7 @@ envfrom(SMFICTX *ctx, char **from)
 
     /* Try making a directory whose name contains the Sendmail qid first */
     if (data->qid && data->qid != NOQUEUE) {
-	snprintf(buffer, SMALLBUF, "%s/mdefang-%s", SPOOLDIR, data->qid);
+	snprintf(buffer, SMALLBUF, "%s/mdefang-%s", SpoolDir, data->qid);
 	if (!mkdir(buffer, 0750)) {
 	    success = 1;
 	}
@@ -652,7 +714,7 @@ envfrom(SMFICTX *ctx, char **from)
 
 	for (tries = 0; tries < MAXTRIES; tries++) {
 	    snprintf(buffer, SMALLBUF, "%s/mdefang-%lX-%d",
-		     SPOOLDIR,
+		     SpoolDir,
 		     ulnow, tries);
 	    if (!mkdir(buffer, 0750)) {
 		success = 1;
@@ -765,7 +827,7 @@ envfrom(SMFICTX *ctx, char **from)
 	int n = MXSenderOK(MultiplexorSocketName, buf2,
 			   (char const **) from, data->hostip, data->hostname,
 			   data->heloArg, data->dir, data->qid);
-	if (n == 0) {
+	if (n == MD_REJECT) {
 	    set_dsn(ctx, buf2, 5);
 
 	    /* We reject connections from this sender */
@@ -773,24 +835,24 @@ envfrom(SMFICTX *ctx, char **from)
 	    DEBUG_EXIT("envfrom", __LINE__, "SMFIS_REJECT");
 	    return SMFIS_REJECT;
 	}
-	if (n < 0) {
+	if (n <= MD_TEMPFAIL) {
 	    set_dsn(ctx, buf2, 4);
 
 	    cleanup(ctx);
 	    DEBUG_EXIT("envfrom", __LINE__, "SMFIS_TEMPFAIL");
 	    return SMFIS_TEMPFAIL;
 	}
-	if (n == 2) {
+	if (n == MD_ACCEPT_AND_NO_MORE_FILTERING) {
 	    set_dsn(ctx, buf2, 2);
 	    cleanup(ctx);
 	    return SMFIS_ACCEPT;
 	}
-	if (n == 3) {
+	if (n == MD_DISCARD) {
 	    set_dsn(ctx, buf2, 2);
 	    cleanup(ctx);
 	    return SMFIS_DISCARD;
 	}
-	if (n == 1) {
+	if (n == MD_CONTINUE) {
 	    /* Called only in case we need to delay */
 	    set_dsn(ctx, buf2, 2);
 	}
@@ -865,31 +927,31 @@ rcptto(SMFICTX *ctx, char **to)
 			  data->hostname, data->firstRecip, data->heloArg,
 			  data->dir, data->qid,
 			  rcpt_mailer, rcpt_host, rcpt_addr);
-	if (n == 0) {
+	if (n == MD_REJECT) {
 	    /* We reject to this recipient */
 	    set_dsn(ctx, ans, 5);
 
 	    DEBUG_EXIT("rcptto", __LINE__, "SMFIS_REJECT");
 	    return SMFIS_REJECT;
 	}
-	if (n < 0) {
+	if (n <= MD_TEMPFAIL) {
 	    set_dsn(ctx, ans, 4);
 
 	    DEBUG_EXIT("rcptto", __LINE__, "SMFIS_TEMPFAIL");
 	    return SMFIS_TEMPFAIL;
 	}
-	if (n == 2) {
+	if (n == MD_ACCEPT_AND_NO_MORE_FILTERING) {
 	    set_dsn(ctx, ans, 2);
-
-	    retcode = SMFIS_ACCEPT;
+	    cleanup(ctx);
+	    return SMFIS_ACCEPT;
 	}
-	if (n == 3) {
+	if (n == MD_DISCARD) {
 	    set_dsn(ctx, ans, 2);
 
 	    cleanup(ctx);
 	    return SMFIS_DISCARD;
 	}
-	if (n == 1) {
+	if (n == MD_CONTINUE) {
 	    /* Called only in case we need to delay */
 	    set_dsn(ctx, ans, 2);
 	}
@@ -1107,6 +1169,11 @@ body(SMFICTX *ctx, u_char *text, size_t len)
 	/* If last was CR, and this is not LF, suspicious! */
 	if (data->lastWasCR && *text != '\n') {
 	    data->suspiciousBody = 1;
+	    if (!StripBareCR) {
+		/* Do not suppress bare CR's.  Only suppress those
+		   followed by LF */
+		buf[nsaved++] = '\r';
+	    }
 	}
 
 	data->lastWasCR = 0;
@@ -1121,10 +1188,17 @@ body(SMFICTX *ctx, u_char *text, size_t len)
 	    if (*s == '\r') {
 		if (n == len-1) {
 		    data->lastWasCR = 1;
+		    continue;
 		} else if (*(s+1) != '\n') {
 		    data->suspiciousBody = 1;
+		    if (StripBareCR) {
+			/* Suppress ALL CR's */
+			continue;
+		    }
+		} else {
+		    /* Suppress the CR immediately preceding a LF */
+		    continue;
 		}
-		continue;
 	    }
 
 	    /* Write char */
@@ -1405,6 +1479,27 @@ eom(SMFICTX *ctx)
 	    }
 	    break;
 
+	case 'N':
+	    /* Insert a header in position count */
+	    split_on_space3(rptr + 1, &hdr, &count, &val);
+	    if (hdr && val && count) {
+		percent_decode((unsigned char *) hdr);
+		percent_decode((unsigned char *) count);
+		percent_decode((unsigned char *) val);
+		if (sscanf(count, "%d", &j) != 1 || j < 0) {
+		    j = 0; /* 0 means add header at the top */
+		}
+#ifdef SMFIR_INSHEADER
+		MD_SMFI_TRY(smfi_insheader, (ctx, j, hdr, val));
+#else
+		syslog(LOG_WARNING,
+		       "%s: No smfi_insheader; using smfi_addheader instead.",
+		       data->qid);
+
+		MD_SMFI_TRY(smfi_addheader, (ctx, hdr, val));
+#endif
+	    }
+	    break;
 	case 'I':
 	    /* Change a header */
 	    split_on_space3(rptr+1, &hdr, &count, &val);
@@ -1679,14 +1774,15 @@ usage(void)
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "  -h                -- Print usage info and exit\n");
     fprintf(stderr, "  -v                -- Print version and exit\n");
-    fprintf(stderr, "  -U user           -- Run as user instead of root\n");
+    fprintf(stderr, "  -m /path          -- Use /path as UNIX-domain socket for multiplexor\n");
     fprintf(stderr, "  -p /path          -- Path to UNIX-domain socket for sendmail communication\n");
+    fprintf(stderr, "  -U user           -- Run as user instead of root\n");
     fprintf(stderr, "  -d                -- Enable debugging (do not remove spool files)\n");
     fprintf(stderr, "  -k                -- Do not remove spool files if filter fails\n");
-    fprintf(stderr, "  -m /path          -- Use multiplexor; use /path as UNIX-domain socket\n");
-    fprintf(stderr, "  -r                -- Do relay check before processing body (requires -m)\n");
-    fprintf(stderr, "  -s                -- Do sender check before processing body (requires -m)\n");
-    fprintf(stderr, "  -t                -- Do recipient checks before processing body (requires -m)\n");
+    fprintf(stderr, "  -z dir            -- Spool directory\n");
+    fprintf(stderr, "  -r                -- Do relay check before processing body\n");
+    fprintf(stderr, "  -s                -- Do sender check before processing body\n");
+    fprintf(stderr, "  -t                -- Do recipient checks before processing body\n");
     fprintf(stderr, "  -q                -- Allow new connections to be queued by multiplexor\n");
     fprintf(stderr, "  -P file           -- Write process-ID of daemon to specified file\n");
     fprintf(stderr, "  -T                -- Log filter times to syslog\n");
@@ -1699,6 +1795,9 @@ usage(void)
     fprintf(stderr, "  -S facility       -- Set syslog(3) facility\n");
     fprintf(stderr, "  -a macro          -- Pass additional Sendmail macro\n");
     fprintf(stderr, "  -L ip.addr        -- Specify 'equivalent-to-loopback' address\n");
+    fprintf(stderr, "  -H                -- Do HELO checks before processing any messages\n");
+    fprintf(stderr, "  -c                -- Strip bare <CR> characters from message body\n");
+    fprintf(stderr, "  -R num            -- Reserve num slaves for connections from localhost\n");
     exit(EXIT_FAILURE);
 }
 
@@ -1766,8 +1865,27 @@ main(int argc, char **argv)
     }
 
     /* Process command line options */
-    while ((c = getopt(argc, argv, "a:Chp:dm:srtkP:U:Tx:MXS:Dvb:L:q")) != -1) {
+    while ((c = getopt(argc, argv, "CDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:")) != -1) {
 	switch (c) {
+	case 'R':
+	    sscanf(optarg, "%d", &slavesReservedForLoopback);
+	    if (slavesReservedForLoopback < -1) {
+		slavesReservedForLoopback = -1;
+	    }
+	    break;
+	case 'H':
+	    doHeloCheck = 1;
+	    break;
+	case 'c':
+	    StripBareCR = 1;
+	    break;
+	case 'z':
+	    SpoolDir = strdup(optarg);
+	    if (!SpoolDir) {
+		fprintf(stderr, "%s: Out of memory\n", argv[0]);
+		exit(EXIT_FAILURE);
+	    }
+	    break;
 	case 'q':
 	    AllowNewConnectionsToQueue = 1;
 	    break;
@@ -1903,8 +2021,8 @@ main(int argc, char **argv)
 
 	case 'p':
 	    if (optarg == NULL || *optarg == '\0') {
-		(void) fprintf(stderr, "%s: Illegal conn: %s\n",
-			       argv[0], optarg);
+		fprintf(stderr, "%s: Illegal conn: %s\n",
+			argv[0], optarg);
 		exit(EXIT_FAILURE);
 	    }
 	    /* Remove socket from file system */
@@ -1921,6 +2039,20 @@ main(int argc, char **argv)
 	}
     }
 
+    /* Set SpoolDir if it wasn't set on command line */
+    if (!SpoolDir) {
+	SpoolDir = SPOOLDIR;
+    }
+    if (!NoDeleteDir) {
+	NoDeleteDir = malloc(strlen(SpoolDir) + strlen(NO_DELETE_NAME) + 1);
+	if (!NoDeleteDir) {
+		fprintf(stderr, "%s: Out of memory\n", argv[0]);
+		exit(EXIT_FAILURE);
+	}
+	strcpy((char *) NoDeleteDir, SpoolDir);
+	strcat((char *) NoDeleteDir, NO_DELETE_NAME);
+    }
+
     if (!scan_body) {
 	scan_body = SCAN_BODY;
     }
@@ -1958,9 +2090,9 @@ main(int argc, char **argv)
     }
 
 
-    if (chdir(SPOOLDIR) < 0) {
+    if (chdir(SpoolDir) < 0) {
 	fprintf(stderr, "%s: Unable to chdir(%s): %s\n",
-		argv[0], SPOOLDIR, strerror(errno));
+		argv[0], SpoolDir, strerror(errno));
 	exit(EXIT_FAILURE);
     }
 
@@ -2022,6 +2154,8 @@ main(int argc, char **argv)
     if (ValidateHeader[0]) {
 	syslog(LOG_DEBUG, "IP validation header is %s", ValidateHeader);
     }
+    syslog(LOG_INFO, "MIMEDefang alive. slavesReservedForLoopback=%d AllowNewConnectionsToQueue=%d doRelayCheck=%d doHeloCheck=%d doSenderCheck=%d doRecipientCheck=%d", slavesReservedForLoopback, AllowNewConnectionsToQueue, doRelayCheck, doHeloCheck, doSenderCheck, doRecipientCheck);
+
 #ifdef ENABLE_DEBUGGING
     signal(SIGSEGV, handle_sig);
     signal(SIGBUS, handle_sig);
@@ -2102,11 +2236,11 @@ remove_working_directory(struct privdata *data)
 	return;
     }
 
-    if (access(NO_DELETE_DIR, F_OK) == 0) {
+    if (access(NoDeleteDir, F_OK) == 0) {
 	syslog(LOG_INFO, "%s: Not cleaning up %s because of %s",
 	       data->qid,
 	       (data->dir ? data->dir : ""),
-	       NO_DELETE_DIR);
+	       NoDeleteDir);
 	return;
     }
 
