@@ -8,7 +8,7 @@
 *
 ***********************************************************************/
 
-/* $Id: mimedefang.h,v 1.29 2005/02/08 16:04:39 dfs Exp $ */
+/* $Id: mimedefang.h,v 1.32 2006/02/01 17:30:53 dfs Exp $ */
 
 #ifndef MIMEDEFANG_H
 #define MIMEDEFANG_H 1
@@ -16,7 +16,8 @@
 #define SMALLBUF 8192
 #define BIGBUF 65536
 #include <stddef.h>
-
+#include <sys/types.h>
+#include <sys/socket.h>   /* For struct sockaddr */
 extern void chomp(char *str);
 extern void write_percent_encoded(unsigned char const *str, int fd);
 extern int percent_encode(unsigned char *in, unsigned char *out, int outlen);
@@ -27,6 +28,8 @@ extern int MXScanDir(char const *sockname, char const *dir);
 extern int MXCommand(char const *sockname, char const *cmd, char *buf, int len);
 extern int MXRelayOK(char const *sockname, char *msg,
 		     char const *ip, char const *name);
+extern int MXHeloOK(char const *sockname, char *msg,
+		    char const *helo, char const *ip, char const *name);
 extern int MXSenderOK(char const *sockname, char *msg,
 		      char const **sender_argv, char const *ip, char const *name,
 		      char const *helo, char const *dir, char const *qid);
@@ -58,13 +61,18 @@ extern int validate_smtp_dsn(char const *dsn, char const *first);
 extern int make_listening_socket(char const *str, int backlog, int must_be_unix);
 extern int connect_to_socket(char const *str);
 extern void do_delay(char const *sleepstr);
-
+extern int is_localhost(struct sockaddr *);
 #ifdef EMBED_PERL
 extern int make_embedded_interpreter(char const *progPath,
 				     char const *subFilter,
 				     int wantStatusReports);
 extern void run_embedded_filter(void);
 #endif
-
+/* Magic return values */
+#define MD_TEMPFAIL                    -1
+#define MD_REJECT                       0
+#define MD_CONTINUE                     1
+#define MD_ACCEPT_AND_NO_MORE_FILTERING 2
+#define MD_DISCARD                      3
 #endif
 
