@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: utils.c,v 1.45 2005/02/08 16:04:40 dfs Exp $";
+"$Id: utils.c,v 1.48 2006/01/20 02:25:51 dfs Exp $";
 
 #define _BSD_SOURCE 1
 
@@ -237,10 +237,7 @@ chomp(char *str)
 *  Note that "str" MAY BE modified!
 * %DESCRIPTION:
 *  Writes "str" to FP with the following changes:
-*    CRLF -> written as-is
-*    LFCR -> written as CRLF
-*    CR   -> written as CRLF
-*    LF   -> written as CRLF
+*    CR   -> written as space
 ***********************************************************************/
 int
 safeWriteHeader(int fd,
@@ -289,7 +286,7 @@ MXCommand(char const *sockname,
     fd = socket(AF_LOCAL, SOCK_STREAM, 0);
     if (fd < 0) {
 	syslog(LOG_ERR, "MXCommand: socket: %m");
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     memset(&addr, 0, sizeof(addr));
@@ -299,14 +296,14 @@ MXCommand(char const *sockname,
     if (connect(fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
 	syslog(LOG_ERR, "MXCommand: connect: %m: Is multiplexor running?");
 	close(fd);
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     n = writestr(fd, cmd);
     if (n < 0) {
 	syslog(LOG_ERR, "MXCommand: write: %m: Is multiplexor running?");
 	close(fd);
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     /* Now read the answer */
@@ -314,7 +311,7 @@ MXCommand(char const *sockname,
     if (nread < 0) {
 	syslog(LOG_ERR, "MXCommand: read: %m: Is multiplexor running?");
 	close(fd);
-	return -1;
+	return MD_TEMPFAIL;
     }
     buf[nread] = 0;
     close(fd);
@@ -337,9 +334,9 @@ MXCheckFreeSlaves(char const *sockname)
     char ans[SMALLBUF];
     int slaves;
 
-    if (MXCommand(sockname, "free\n", ans, SMALLBUF-1) < 0) return -1;
+    if (MXCommand(sockname, "free\n", ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
 
-    if (sscanf(ans, "%d", &slaves) != 1) return -1;
+    if (sscanf(ans, "%d", &slaves) != 1) return MD_TEMPFAIL;
     return slaves;
 }
 
@@ -362,14 +359,14 @@ MXScanDir(char const *sockname,
     int len;
 
     snprintf(cmd, SMALLBUF, "scan %s\n", dir);
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
 
     if (!strcmp(ans, "ok\n")) return 0;
 
     len = strlen(ans);
     if (len > 0 && ans[len-1] == '\n') ans[len-1] = 0;
     syslog(LOG_ERR, "Error from multiplexor: %s", ans);
-    return -1;
+    return MD_TEMPFAIL;
 }
 
 /**********************************************************************
@@ -529,7 +526,7 @@ percent_encode_command(int term_with_newline, char *out, int outlen, ...)
     int len;
     char *arg;
 
-    if (outlen < 2) return -1;
+    if (outlen < 2) return MD_TEMPFAIL;
 
     va_start(ap, outlen);
 
@@ -539,7 +536,7 @@ percent_encode_command(int term_with_newline, char *out, int outlen, ...)
 	} else {
 	    if (spaceleft <= 0) {
 		va_end(ap);
-		return -1;
+		return MD_TEMPFAIL;
 	    }
 	    *out++ = ' ';
 	    spaceleft--;
@@ -569,11 +566,11 @@ munch_mx_return(char *ans, char *msg)
 {
     size_t len;
 
-    if (!strcmp(ans, "ok -1\n")) return -1;
-    if (!strcmp(ans, "ok 1\n")) return 1;
-    if (!strcmp(ans, "ok 2\n")) return 2;
-    if (!strcmp(ans, "ok 3\n")) return 3;
-    if (!strcmp(ans, "ok 0\n")) return 0;
+    if (!strcmp(ans, "ok -1\n")) return MD_TEMPFAIL;
+    if (!strcmp(ans, "ok 1\n")) return MD_CONTINUE;
+    if (!strcmp(ans, "ok 2\n")) return MD_ACCEPT_AND_NO_MORE_FILTERING;
+    if (!strcmp(ans, "ok 3\n")) return MD_DISCARD;
+    if (!strcmp(ans, "ok 0\n")) return MD_REJECT;
 
     chomp(ans);
 
@@ -581,32 +578,32 @@ munch_mx_return(char *ans, char *msg)
     len = strlen(ans);
     if (len >= 6 && !strncmp(ans, "ok 0 ", 5)) {
 	strcpy(msg, ans+5);
-	return 0;
+	return MD_REJECT;
     }
 
     if (len >= 7 && !strncmp(ans, "ok -1 ", 6)) {
 	strcpy(msg, ans+6);
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     if (len >= 6 && !strncmp(ans, "ok 1 ", 5)) {
 	strcpy(msg, ans+5);
-	return 1;
+	return MD_CONTINUE;
     }
 
     if (len >= 6 && !strncmp(ans, "ok 2 ", 5)) {
 	strcpy(msg, ans+5);
-	return 2;
+	return MD_ACCEPT_AND_NO_MORE_FILTERING;
     }
 
     if (len >= 6 && !strncmp(ans, "ok 3 ", 5)) {
 	strcpy(msg, ans+5);
-	return 3;
+	return MD_DISCARD;
     }
 
     if (len > 0 && ans[len-1] == '\n') ans[len-1] = 0;
     syslog(LOG_ERR, "Error from multiplexor: %s", ans);
-    return -1;
+    return MD_TEMPFAIL;
 }
 
 /**********************************************************************
@@ -638,12 +635,51 @@ MXRelayOK(char const *sockname,
 	name = ip;
     }
     if (percent_encode_command(1, cmd, sizeof(cmd), "relayok", ip, name, NULL) < 0) {
-	return -1;
+	return MD_TEMPFAIL;
     }
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
     return munch_mx_return(ans, msg);
 }
 
+/**********************************************************************
+* %FUNCTION: MXHeloOK
+* %ARGUMENTS:
+*  sockname -- multiplexor socket name
+*  msg -- buffer for holding error message, at least SMALLBUF chars
+*  helo -- the helo string
+* %RETURNS:
+*  1 if it's OK to accept messages from this sender; 0 if not, -1 if error or
+*  we should tempfail.
+***********************************************************************/
+int
+MXHeloOK(char const *sockname,
+	 char *msg,
+	 char const *ip,
+	 char const *name,
+	 char const *helo)
+{
+    char cmd[SMALLBUF];
+    char ans[SMALLBUF];
+
+    *msg = 0;
+
+    if (!ip || !*ip) {
+	ip = "UNKNOWN";
+    }
+    if (!name || !*name) {
+	name = ip;
+    }
+    if (!helo) {
+	helo = "UNKNOWN";
+    }
+    if (percent_encode_command(1, cmd, sizeof(cmd), "helook", ip, name, helo, NULL) < 0) {
+	return MD_TEMPFAIL;
+    }
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
+    return munch_mx_return(ans, msg);
+}
+
+
 /**********************************************************************
 * %FUNCTION: MXSenderOK
 * %ARGUMENTS:
@@ -695,7 +731,7 @@ MXSenderOK(char const *sockname,
     if (percent_encode_command(0, cmd, sizeof(cmd)-1, "senderok", sender, ip,
 			       name,
 			       helo, dir, qid, NULL) < 0) {
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     /* Append ESMTP args */
@@ -717,7 +753,7 @@ MXSenderOK(char const *sockname,
     /* Add newline */
     strcat(cmd, "\n");
 
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
     return munch_mx_return(ans, msg);
 }
 
@@ -790,7 +826,7 @@ MXRecipientOK(char const *sockname,
 			       "recipok", recipient, sender, ip, name, firstRecip,
 			       helo, dir, qid, rcpt_mailer, rcpt_host, rcpt_addr,
 			       NULL) < 0) {
-	return -1;
+	return MD_TEMPFAIL;
     }
 
     /* Append ESMTP args */
@@ -812,7 +848,7 @@ MXRecipientOK(char const *sockname,
     /* Add newline */
     strcat(cmd, "\n");
 
-    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return -1;
+    if (MXCommand(sockname, cmd, ans, SMALLBUF-1) < 0) return MD_TEMPFAIL;
     return munch_mx_return(ans, msg);
 }
 
@@ -869,7 +905,7 @@ writen(int fd,
 	if (r == 0) {
 	    /* Shouldn't happen! */
 	    errno = EIO;
-	    return -1;
+	    return MD_TEMPFAIL;
 	}
 	if (r < 0) {
 	    if (errno == EINTR || errno == EAGAIN) {
@@ -1225,3 +1261,26 @@ do_delay(char const *sleepstr)
 	snooze = sleep(snooze);
     }
 }
+
+/**********************************************************************
+* %FUNCTION: is_localhost
+* %ARGUMENTS:
+*  sa -- a socket address
+* %RETURNS:
+*  True if sa is the loobpack address; false otherwise.
+***********************************************************************/
+int
+is_localhost(struct sockaddr *sa)
+{
+    if (sa->sa_family == AF_INET) {
+	struct sockaddr_in *sa_in = (struct sockaddr_in *) sa;
+	return (sa_in->sin_addr.s_addr == htonl(INADDR_LOOPBACK));
+    }
+#ifdef AF_INET6
+    if (sa->sa_family == AF_INET6) {
+	struct sockaddr_in6 *sa_in6 = (struct sockaddr_in6 *) sa;
+	return IN6_IS_ADDR_LOOPBACK(&sa_in6->sin6_addr);
+    }
+#endif
+    return 0;
+}
